@@ -5,8 +5,35 @@ use std::str::FromStr;
 use video_timecode::*;
 
 // Test creating timecodes with various frame rates and durations.
+//
+// To check which frame number corresponds to which timecode in drop frame frame
+// rates, I had this python script write them all out to a file
+//
+//     i = 0;
+//     for h in range(0, 24):
+//         for m in range(0, 60):
+//             for s in range(0, 60):
+//                 for f in range(0, 30):  # 60 for 59.94
+//                     if f < 2 and s == 0 and m % 10 != 0:  # f < 4 for 59.94
+//                         continue
+//                     print("{:02d}:{:02d}:{:02d};{:02d} {}".format(h, m, s, f, i))
+//                     i += 1
 
-macro_rules! test_frame_number {
+macro_rules! assert_tc {
+    ($tc:expr,
+     $hour:expr,
+     $minute:expr,
+     $second:expr,
+     $frame:expr,
+     $frame_number:expr) => (
+         assert_eq!(
+             ($tc.hour, $tc.minute, $tc.second, $tc.frame, $tc.frame_number),
+             ($hour, $minute, $second, $frame, $frame_number)
+         );
+    )
+}
+
+macro_rules! test_factories {
     ($name:ident,
      $hour:expr,
      $minute:expr,
@@ -16,40 +43,21 @@ macro_rules! test_frame_number {
      $frame_number:expr) => (
          #[test]
          fn $name() {
-            assert_eq!(
-                Timecode::<$frame_rate>::new($hour,
-                                             $minute,
-                                             $second,
-                                             $frame)
-                    .unwrap()
-                    .frame_number,
-                $frame_number
-            )
+            let tc_new = Timecode::<$frame_rate>::new($hour, $minute, $second, $frame)
+                .unwrap();
+            assert_tc!(tc_new, $hour, $minute, $second, $frame, $frame_number);
+
+            let tc_from = Timecode::<$frame_rate>::from($frame_number);
+            assert_tc!(tc_from, $hour, $minute, $second, $frame, $frame_number);
          }
     )
 }
 
-test_frame_number!(test_frame_number_24_zero, 0, 0, 0, 0, FrameRate24, 0);
-test_frame_number!(
-    test_frame_number_24_with_frames,
-    0,
-    0,
-    0,
-    20,
-    FrameRate24,
-    20
-);
-test_frame_number!(
-    test_frame_number_24_with_seconds,
-    0,
-    0,
-    40,
-    10,
-    FrameRate24,
-    970
-);
-test_frame_number!(
-    test_frame_number_24_with_minutes,
+test_factories!(frame_number_24_zero, 0, 0, 0, 0, FrameRate24, 0);
+test_factories!(frame_number_24_with_frames, 0, 0, 0, 20, FrameRate24, 20);
+test_factories!(frame_number_24_with_seconds, 0, 0, 40, 10, FrameRate24, 970);
+test_factories!(
+    frame_number_24_with_minutes,
     0,
     23,
     30,
@@ -57,8 +65,8 @@ test_frame_number!(
     FrameRate24,
     33855
 );
-test_frame_number!(
-    test_frame_number_24_with_hours,
+test_factories!(
+    frame_number_24_with_hours,
     5,
     15,
     25,
@@ -67,18 +75,10 @@ test_frame_number!(
     454212
 );
 
-test_frame_number!(test_frame_number_25_zero, 0, 0, 0, 0, FrameRate25, 0);
-test_frame_number!(
-    test_frame_number_25_with_frames,
-    0,
-    0,
-    0,
-    23,
-    FrameRate25,
-    23
-);
-test_frame_number!(
-    test_frame_number_25_with_seconds,
+test_factories!(frame_number_25_zero, 0, 0, 0, 0, FrameRate25, 0);
+test_factories!(frame_number_25_with_frames, 0, 0, 0, 23, FrameRate25, 23);
+test_factories!(
+    frame_number_25_with_seconds,
     0,
     0,
     40,
@@ -86,8 +86,8 @@ test_frame_number!(
     FrameRate25,
     1010
 );
-test_frame_number!(
-    test_frame_number_25_with_minutes,
+test_factories!(
+    frame_number_25_with_minutes,
     0,
     23,
     30,
@@ -95,8 +95,8 @@ test_frame_number!(
     FrameRate25,
     35265
 );
-test_frame_number!(
-    test_frame_number_25_with_hours,
+test_factories!(
+    frame_number_25_with_hours,
     5,
     15,
     25,
@@ -105,18 +105,10 @@ test_frame_number!(
     473137
 );
 
-test_frame_number!(test_frame_number_30_zero, 0, 0, 0, 0, FrameRate30, 0);
-test_frame_number!(
-    test_frame_number_30_with_frames,
-    0,
-    0,
-    0,
-    26,
-    FrameRate30,
-    26
-);
-test_frame_number!(
-    test_frame_number_30_with_seconds,
+test_factories!(frame_number_30_zero, 0, 0, 0, 0, FrameRate30, 0);
+test_factories!(frame_number_30_with_frames, 0, 0, 0, 26, FrameRate30, 26);
+test_factories!(
+    frame_number_30_with_seconds,
     0,
     0,
     40,
@@ -124,8 +116,8 @@ test_frame_number!(
     FrameRate30,
     1210
 );
-test_frame_number!(
-    test_frame_number_30_with_minutes,
+test_factories!(
+    frame_number_30_with_minutes,
     0,
     23,
     30,
@@ -133,8 +125,8 @@ test_frame_number!(
     FrameRate30,
     42315
 );
-test_frame_number!(
-    test_frame_number_30_with_hours,
+test_factories!(
+    frame_number_30_with_hours,
     5,
     15,
     25,
@@ -143,18 +135,10 @@ test_frame_number!(
     567762
 );
 
-test_frame_number!(test_frame_number_50_zero, 0, 0, 0, 0, FrameRate50, 0);
-test_frame_number!(
-    test_frame_number_50_with_frames,
-    0,
-    0,
-    0,
-    35,
-    FrameRate50,
-    35
-);
-test_frame_number!(
-    test_frame_number_50_with_seconds,
+test_factories!(frame_number_50_zero, 0, 0, 0, 0, FrameRate50, 0);
+test_factories!(frame_number_50_with_frames, 0, 0, 0, 35, FrameRate50, 35);
+test_factories!(
+    frame_number_50_with_seconds,
     0,
     0,
     40,
@@ -162,8 +146,8 @@ test_frame_number!(
     FrameRate50,
     2010
 );
-test_frame_number!(
-    test_frame_number_50_with_minutes,
+test_factories!(
+    frame_number_50_with_minutes,
     0,
     23,
     30,
@@ -171,8 +155,8 @@ test_frame_number!(
     FrameRate50,
     70515
 );
-test_frame_number!(
-    test_frame_number_50_with_hours,
+test_factories!(
+    frame_number_50_with_hours,
     5,
     15,
     25,
@@ -181,18 +165,10 @@ test_frame_number!(
     946262
 );
 
-test_frame_number!(test_frame_number_60_zero, 0, 0, 0, 0, FrameRate60, 0);
-test_frame_number!(
-    test_frame_number_60_with_frames,
-    0,
-    0,
-    0,
-    45,
-    FrameRate60,
-    45
-);
-test_frame_number!(
-    test_frame_number_60_with_seconds,
+test_factories!(frame_number_60_zero, 0, 0, 0, 0, FrameRate60, 0);
+test_factories!(frame_number_60_with_frames, 0, 0, 0, 45, FrameRate60, 45);
+test_factories!(
+    frame_number_60_with_seconds,
     0,
     0,
     40,
@@ -200,8 +176,8 @@ test_frame_number!(
     FrameRate60,
     2410
 );
-test_frame_number!(
-    test_frame_number_60_with_minutes,
+test_factories!(
+    frame_number_60_with_minutes,
     0,
     23,
     30,
@@ -209,8 +185,8 @@ test_frame_number!(
     FrameRate60,
     84615
 );
-test_frame_number!(
-    test_frame_number_60_with_hours,
+test_factories!(
+    frame_number_60_with_hours,
     23,
     59,
     59,
@@ -219,9 +195,9 @@ test_frame_number!(
     5183963
 );
 
-test_frame_number!(test_frame_number_2398_zero, 0, 0, 0, 0, FrameRate2398, 0);
-test_frame_number!(
-    test_frame_number_2398_with_frames,
+test_factories!(frame_number_2398_zero, 0, 0, 0, 0, FrameRate2398, 0);
+test_factories!(
+    frame_number_2398_with_frames,
     0,
     0,
     0,
@@ -229,8 +205,8 @@ test_frame_number!(
     FrameRate2398,
     22
 );
-test_frame_number!(
-    test_frame_number_2398_with_seconds,
+test_factories!(
+    frame_number_2398_with_seconds,
     0,
     0,
     40,
@@ -238,8 +214,8 @@ test_frame_number!(
     FrameRate2398,
     970
 );
-test_frame_number!(
-    test_frame_number_2398_with_minutes,
+test_factories!(
+    frame_number_2398_with_minutes,
     0,
     23,
     30,
@@ -247,8 +223,8 @@ test_frame_number!(
     FrameRate2398,
     33855
 );
-test_frame_number!(
-    test_frame_number_2398_with_hours,
+test_factories!(
+    frame_number_2398_with_hours,
     5,
     15,
     25,
@@ -257,9 +233,9 @@ test_frame_number!(
     454212
 );
 
-test_frame_number!(test_frame_number_2997_zero, 0, 0, 0, 0, FrameRate2997, 0);
-test_frame_number!(
-    test_frame_number_2997_with_frames,
+test_factories!(frame_number_2997_zero, 0, 0, 0, 0, FrameRate2997, 0);
+test_factories!(
+    frame_number_2997_with_frames,
     0,
     0,
     0,
@@ -267,8 +243,8 @@ test_frame_number!(
     FrameRate2997,
     20
 );
-test_frame_number!(
-    test_frame_number_2997_with_seconds,
+test_factories!(
+    frame_number_2997_with_seconds,
     0,
     0,
     40,
@@ -276,8 +252,8 @@ test_frame_number!(
     FrameRate2997,
     1210
 );
-test_frame_number!(
-    test_frame_number_2997_with_minutes,
+test_factories!(
+    frame_number_2997_with_minutes,
     0,
     23,
     30,
@@ -285,19 +261,19 @@ test_frame_number!(
     FrameRate2997,
     42273
 );
-test_frame_number!(
-    test_frame_number_2997_with_hours,
-    5,
-    15,
-    25,
-    12,
+test_factories!(
+    frame_number_2997_with_hours,
+    23,
+    23,
+    23,
+    23,
     FrameRate2997,
-    567194
+    2523587
 );
 
-test_frame_number!(test_frame_number_5994_zero, 0, 0, 0, 0, FrameRate5994, 0);
-test_frame_number!(
-    test_frame_number_5994_with_frames,
+test_factories!(frame_number_5994_zero, 0, 0, 0, 0, FrameRate5994, 0);
+test_factories!(
+    frame_number_5994_with_frames,
     0,
     0,
     0,
@@ -305,8 +281,8 @@ test_frame_number!(
     FrameRate5994,
     45
 );
-test_frame_number!(
-    test_frame_number_5994_with_seconds,
+test_factories!(
+    frame_number_5994_with_seconds,
     0,
     0,
     40,
@@ -314,8 +290,8 @@ test_frame_number!(
     FrameRate5994,
     2410
 );
-test_frame_number!(
-    test_frame_number_5994_with_minutes,
+test_factories!(
+    frame_number_5994_with_minutes,
     0,
     23,
     30,
@@ -323,8 +299,8 @@ test_frame_number!(
     FrameRate5994,
     84531
 );
-test_frame_number!(
-    test_frame_number_5994_with_hours,
+test_factories!(
+    frame_number_5994_with_hours,
     5,
     15,
     25,
@@ -336,175 +312,159 @@ test_frame_number!(
 // Test adding integers to Timecodes
 
 #[test]
-fn test_add_frames_1() {
+fn add_frames_1() {
     let tc = Timecode::<FrameRate24>::new(0, 0, 0, 0).unwrap() + 1usize;
-    assert_eq!(
-        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
-        (0, 0, 0, 1, 1)
-    );
+    assert_tc!(tc, 0, 0, 0, 1, 1);
 }
 
 #[test]
-fn test_add_frames_500() {
+fn add_frames_500() {
     let tc = Timecode::<FrameRate24>::new(0, 0, 0, 0).unwrap() + 500u32;
-    assert_eq!(
-        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
-        (0, 0, 20, 20, 500)
-    );
+    assert_tc!(tc, 0, 0, 20, 20, 500);
 }
 
 #[test]
-fn test_add_frames_1000_df() {
+fn add_frames_1000_df() {
     let tc = Timecode::<FrameRate2997>::new(10, 0, 0, 0).unwrap() + 1000u32;
-    assert_eq!(
-        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
-        (10, 0, 33, 10, 1079920)
-    );
+    assert_tc!(tc, 10, 0, 33, 10, 1079920);
 }
 
 #[test]
-fn test_add_frames_1000000_df() {
+fn add_frames_1000000_df() {
     let tc = Timecode::<FrameRate2997>::new(10, 0, 0, 0).unwrap() + 1000000u32;
-    assert_eq!(
-        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
-        (19, 16, 6, 22, 2078920)
-    );
+    assert_tc!(tc, 19, 16, 6, 22, 2078920);
 }
 
 #[test]
-fn test_add_frames_negative_2997() {
+fn add_frames_negative_2997() {
     let tc = Timecode::<FrameRate2997>::new(0, 0, 10, 0).unwrap() + (-1000i32);
-    assert_eq!(
-        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
-        (23, 59, 36, 20, 2588708)
-    );
+    assert_tc!(tc, 23, 59, 36, 20, 2588708);
 }
 
 #[test]
-fn test_add_frames_negative_5994() {
+fn add_frames_negative_5994() {
     let tc = Timecode::<FrameRate5994>::new(0, 0, 10, 0).unwrap() + (-1000i32);
-    assert_eq!(
-        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
-        (23, 59, 53, 20, 5178416)
-    );
+    assert_tc!(tc, 23, 59, 53, 20, 5178416);
 }
 
 // Test adding to timecodes with += operator
 
 #[test]
-fn test_add_assign_frames_1() {
+fn add_assign_frames_1() {
     let mut tc = Timecode::<FrameRate24>::new(0, 0, 0, 0).unwrap();
     tc += 1usize;
-    assert_eq!(
-        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
-        (0, 0, 0, 1, 1)
-    );
+    assert_tc!(tc, 0, 0, 0, 1, 1);
 }
 
 #[test]
-fn test_add_assign_frames_500() {
+fn add_assign_frames_500() {
     let mut tc = Timecode::<FrameRate24>::new(0, 0, 0, 0).unwrap();
     tc += 500u32;
-    assert_eq!(
-        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
-        (0, 0, 20, 20, 500)
-    );
+    assert_tc!(tc, 0, 0, 20, 20, 500);
 }
 
 #[test]
-fn test_add_assign_frames_1000_df() {
+fn add_assign_frames_1000_df() {
     let mut tc = Timecode::<FrameRate2997>::new(10, 0, 0, 0).unwrap();
     tc += 1000u32;
-    assert_eq!(
-        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
-        (10, 0, 33, 10, 1079920)
-    );
+    assert_tc!(tc, 10, 0, 33, 10, 1079920);
 }
 
 #[test]
-fn test_add_assign_frames_negative_2997() {
+fn add_assign_frames_negative_2997() {
     let mut tc = Timecode::<FrameRate2997>::new(0, 0, 10, 0).unwrap();
     tc += -1000i32;
-    assert_eq!(
-        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
-        (23, 59, 36, 20, 2588708)
-    );
+    assert_tc!(tc, 23, 59, 36, 20, 2588708);
 }
 
 #[test]
-fn test_add_assign_frames_negative_5994() {
+fn add_assign_frames_negative_5994() {
     let mut tc = Timecode::<FrameRate5994>::new(0, 0, 10, 0).unwrap();
     tc += -1000i32;
-    assert_eq!(
-        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
-        (23, 59, 53, 20, 5178416)
-    );
+    assert_tc!(tc, 23, 59, 53, 20, 5178416);
 }
 
 // Test adding Timecodes to Timecodes
 
 #[test]
-fn test_add_timecode_1() {
+fn add_timecode_1() {
     let tc = Timecode::<FrameRate50>::new(0, 0, 0, 0).unwrap()
         + Timecode::<FrameRate50>::new(0, 0, 0, 1).unwrap();
     assert_eq!(tc, Timecode::<FrameRate50>::new(0, 0, 0, 1).unwrap());
 }
 
 #[test]
-fn test_add_timecode_500() {
+fn add_timecode_500() {
     let tc = Timecode::<FrameRate24>::new(0, 0, 0, 10).unwrap()
         + Timecode::<FrameRate24>::new(0, 0, 20, 10).unwrap();
     assert_eq!(tc, Timecode::<FrameRate24>::new(0, 0, 20, 20).unwrap());
 }
 
 #[test]
-fn test_add_timecode_1000_df() {
+fn add_timecode_1000_df() {
     let tc = Timecode::<FrameRate2997>::new(10, 0, 0, 0).unwrap()
         + Timecode::<FrameRate2997>::new(0, 0, 33, 10).unwrap();
     assert_eq!(tc, Timecode::<FrameRate2997>::new(10, 0, 33, 10).unwrap());
 }
 
 #[test]
-fn test_add_timecode_1000000_df() {
+fn add_timecode_1000000_df() {
     let tc = Timecode::<FrameRate2997>::new(10, 0, 0, 0).unwrap()
         + Timecode::<FrameRate2997>::new(9, 16, 6, 22).unwrap();
     assert_eq!(tc, Timecode::<FrameRate2997>::new(19, 16, 6, 22).unwrap());
 }
 
+// Test subtracting integers from Timecodes
+
+#[test]
+fn subtract_frames_1() {
+    let tc = Timecode::<FrameRate24>::new(0, 0, 0, 10).unwrap() - 1usize;
+    assert_eq!(
+        (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
+        (0, 0, 0, 9, 9)
+    );
+}
+
+#[test]
+fn sub_frames_500() {
+    let tc = Timecode::<FrameRate24>::new(0, 0, 20, 20).unwrap() - 500u32;
+    assert_tc!(tc, 0, 0, 0, 0, 0);
+}
+
 // Test parsing
 
 #[test]
-fn test_parse_0() {
+fn parse_0() {
     let tc = Timecode::<FrameRate24>::from_str("00:00:00:00").unwrap();
     assert_eq!(tc, Timecode::<FrameRate24>::new(0, 0, 0, 0).unwrap());
 }
 
 #[test]
-fn test_parse_0_df_one_semicolon() {
+fn parse_0_df_one_semicolon() {
     let tc = Timecode::<FrameRate2997>::from_str("00:00:00;00").unwrap();
     assert_eq!(tc, Timecode::<FrameRate2997>::new(0, 0, 0, 0).unwrap());
 }
 
 #[test]
-fn test_parse_0_df_all_semicolon() {
+fn parse_0_df_all_semicolon() {
     let tc = Timecode::<FrameRate2997>::from_str("00;00;00;00").unwrap();
     assert_eq!(tc, Timecode::<FrameRate2997>::new(0, 0, 0, 0).unwrap());
 }
 
 #[test]
-fn test_parse_0_df_one_dot() {
+fn parse_0_df_one_dot() {
     let tc = Timecode::<FrameRate2997>::from_str("00:00:00.00").unwrap();
     assert_eq!(tc, Timecode::<FrameRate2997>::new(0, 0, 0, 0).unwrap());
 }
 
 #[test]
-fn test_parse_0_df_all_dot() {
+fn parse_0_df_all_dot() {
     let tc = Timecode::<FrameRate2997>::from_str("00.00.00.00").unwrap();
     assert_eq!(tc, Timecode::<FrameRate2997>::new(0, 0, 0, 0).unwrap());
 }
 
 #[test]
-fn test_parse_0_df_mixed() {
+fn parse_0_df_mixed() {
     match Timecode::<FrameRate2997>::from_str("00.00:00.00") {
         Err(ParseTimecodeError {
             kind: video_timecode::TimecodeErrorKind::InvalidFormat,
@@ -514,7 +474,7 @@ fn test_parse_0_df_mixed() {
 }
 
 #[test]
-fn test_parse_0_df_for_ndf_frame_rate() {
+fn parse_0_df_for_ndf_frame_rate() {
     match Timecode::<FrameRate24>::from_str("00:00:00;00") {
         Err(ParseTimecodeError {
             kind: video_timecode::TimecodeErrorKind::InvalidDropFrameFormat,
@@ -524,13 +484,35 @@ fn test_parse_0_df_for_ndf_frame_rate() {
 }
 
 #[test]
-fn test_parse_500() {
+fn parse_500() {
     let tc = Timecode::<FrameRate24>::from_str("00:00:20:10").unwrap();
     assert_eq!(tc, Timecode::<FrameRate24>::new(0, 0, 20, 10).unwrap());
 }
 
 #[test]
-fn test_parse_1000000_df() {
+fn parse_1000000_df() {
     let tc = Timecode::<FrameRate2997>::from_str("00:00:20:10").unwrap();
     assert_eq!(tc, Timecode::<FrameRate2997>::new(0, 0, 20, 10).unwrap())
 }
+
+// Test formatting
+
+#[test]
+fn format_ndf() {
+    assert_eq!(
+        Timecode::<FrameRate50>::new(10, 12, 6, 00)
+            .unwrap()
+            .to_string(),
+        "10:12:06:00"
+    );
+}
+
+#[test]
+fn format_df() {
+    assert_eq!(
+        Timecode::<FrameRate2997>::new(10, 0, 20, 10)
+            .unwrap()
+            .to_string(),
+        "10:00:20;10"
+    );
+}
